/******************************************************************************
 *
 * dt_memory.c - Memory handling code. Part of the DirTools project.
 *
 * $Revision: 1.1.1.1 $
 *
 * Copyright (c) 2001-2004 Olivier Castan
 *
 * This program/include file is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program/include file is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the Linux-NTFS
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * format : indent -br -ce -cdw -nut -i5
 *
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif /* HAVE_CONFIG_H */

#include <stdlib.h>

#include "dt_memory.h"
#include "dt_error.h"

/**********************************************************************
 * dt_calloc
 *
 *
 *
 *
 *
 *
 *********************************************************************/
void *
dt_calloc (size_t size, size_t number, TYPE_ALARM level,
           const char *errmsg, ...)
{
     va_list args;
     void *pointer;

#ifdef _WIN32
     pointer = HeapAlloc (GetProcessHeap (), HEAP_ZERO_MEMORY, size * number);
#else
     if ((number * size) > 0) {
          pointer = calloc (number, size);
     } else {
          pointer = NULL;
     }
#endif
     if (pointer == NULL && ((number * size) > 0)) {
          va_start (args, errmsg);
          Alarm (level, errmsg, args);
          va_end (args);
     }

     return pointer;
}

/**********************************************************************
 * dt_realloc
 *
 *
 *
 *
 *
 *
 *********************************************************************/
void *
dt_realloc (void *oldp, size_t size, size_t number, TYPE_ALARM level,
            const char *errmsg, ...)
{
     va_list args;
     void *pointer;

#ifdef _WIN32
     if (oldp != NULL)
          pointer = HeapReAlloc (GetProcessHeap (),
                                 HEAP_ZERO_MEMORY, oldp, size * number);
     else
          pointer = HeapAlloc (GetProcessHeap (),
                               HEAP_ZERO_MEMORY, size * number);
#else
     pointer = realloc (oldp, number * size);
#endif
     if (pointer == NULL && ((number * size) > 0)) {
          va_start (args, errmsg);
          Alarm (level, errmsg, args);
          va_end (args);
     }

     return pointer;
}

/**********************************************************************
 * dt_free
 *
 *
 *
 *
 *
 *
 *********************************************************************/
void
dt_free (void *oldp, TYPE_ALARM level, const char *errmsg, ...)
{
#ifdef _WIN32
     va_list args;

     if (!HeapFree (GetProcessHeap (), 0, oldp)) {
          va_start (args, errmsg);
          Alarm (level, errmsg, args);
          va_end (args);
     }
#else
     free (oldp);
#endif
}
